@@ -1,4 +1,3 @@
-static char rcsid[]= "$Id: xlbiff.c,v 1.75 2003/11/08 23:40:18 esm Exp $";
 /*\
 |* xlbiff  --  X Literate Biff
 |*
@@ -10,33 +9,22 @@ static char rcsid[]= "$Id: xlbiff.c,v 1.75 2003/11/08 23:40:18 esm Exp $";
 |*	Xlbiff is modeled after xconsole; it remains invisible at non-
 |*	useful times, eg, when no mail is present.  See README for details.
 |*
-|*	Author:		Eduardo Santiago, esm@pobox.com
+|*	Author:		Eduardo Santiago, ed@edsantiago.com
 |* 	Created:	20 August 1991
-|*	Last Updated:	 8 November 2003
-|*
-|*    Copyright 1994 Eduardo Santiago
-|*
-|*    The X Consortium, and any party obtaining a copy of these files from
-|*    the X Consortium, directly or indirectly, is granted, free of charge, a
-|*    full and unrestricted irrevocable, world-wide, paid up, royalty-free,
-|*    nonexclusive right and license to deal in this software and
-|*    documentation files (the "Software"), including without limitation the
-|*    rights to use, copy, modify, merge, publish, distribute, sublicense,
-|*    and/or sell copies of the Software, and to permit persons who receive
-|*    copies from any such party to do so.  This license includes without
-|*    limitation a license to do the foregoing actions under any patents of
-|*    the party supplying this software to the X Consortium.
+|*	Last Updated:	16 May 2017
 |*
+|*    Copyright 1994, 2017 Eduardo Santiago
+|*    SPDX-License-Identifier: MIT
 \*/
 
-#include "patchlevel.h"
-
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
 #include <X11/Shell.h>
 #include <X11/Xaw/Command.h>
 #include <X11/Xos.h>
 
+#include <unistd.h>
+#include <stdlib.h>
 #include <stdio.h>
 #include <sys/stat.h>
 #include <pwd.h>
@@ -72,10 +60,6 @@ typedef union wait      waitType;
 #endif /* ! X_NOT_POSIX else */
 
 
-#ifdef	NEED_STRERROR
-char	*strerror();
-#endif
-
 /*
 ** if compiled with -DDEBUG *and* run with debugging on, this does lots
 ** of useless/useful printfs.
@@ -90,8 +74,8 @@ char	*strerror();
 ** This defines the file we need to monitor.  If not defined explicitly
 ** on the command line, we pick this default.
 */
-#ifndef	MAILPATH
-#define	MAILPATH	"/usr/spool/mail/%s"
+#ifndef	XLBIFF_MAILPATH
+#define	XLBIFF_MAILPATH	"/var/mail/%s"
 #endif
 
 /*****************************************************************************\
@@ -102,7 +86,6 @@ void		Popdown(),Popup();
 void		Usage();
 extern char	*getlogin();
 
-#ifdef	FUNCPROTO
 void	Shrink(Widget, caddr_t, XEvent*, Boolean*);
 void	handler(XtPointer,XtIntervalId*);
 void	initStaticData(int*,int*,int*);
@@ -113,18 +96,6 @@ void	getDimensions(char*,Dimension*,Dimension*);
 void	toggle_key_led(int);
 void	ErrExit(Boolean,char*);
 Bool	CheckEvent(Display*,XEvent*,caddr_t);
-#else
-void	Shrink();
-void	handler();
-void	initStaticData();
-void	Exit();
-void	Mailer();
-void	lbiffUnrealize(), lbiffRealize();
-void	getDimensions();
-void	toggle_key_led();
-void	ErrExit();
-Bool	CheckEvent();
-#endif
 
 /*****************************************************************************\
 **                                 globals				     **
@@ -222,7 +193,7 @@ static XrmOptionDescRec	optionDescList[] = {
 };
 
 static char *fallback_resources[] = {
-    "*Font:		-*-clean-bold-r-normal--13-130-75-75-c-80-iso8859-1",
+    "*Font:		-*-clean-bold-r-normal--13-130-75-75-c-80-iso646.1991-*",
     "*Geometry:		+0-0",
     NULL
 };
@@ -241,16 +212,12 @@ static XtActionsRec lbiff_actions[] = {
 /**********\
 |*  main  *|
 \**********/
-#ifdef	FUNCPROTO
+int
 main( int argc, char *argv[] )
-#else
-main(argc, argv)
-    int argc;
-    char *argv[];
-#endif
 {
     progname = argv[0];
 
+    XtSetLanguageProc (NULL, NULL, NULL);
     topLevel = XtVaAppInitialize(&app_context,
 				 "XLbiff",
 				 optionDescList, XtNumber(optionDescList),
@@ -273,13 +240,7 @@ main(argc, argv)
     */
     if (argc > 1) {
 	if (!strncmp(argv[1],"-version",strlen(argv[1]))) {
-	    fprintf(stderr,
-#if	TESTLEVEL != 0
-		    "%s version %d.%d.%d\n",
-#else
-		    "%s version %d.%d\n",
-#endif
-		    progname,  VERSION,PATCHLEVEL,TESTLEVEL);
+	    fprintf(stderr, "%s version %s\n", progname, VERSION);
 	    exit(0);
 	} else if (!strncmp(argv[1],"-help",strlen(argv[1]))) {
 	    Usage();
@@ -301,7 +262,7 @@ main(argc, argv)
 
 	if (username == NULL || username[0] == '\0') {
 	    struct passwd  *pwd = getpwuid(getuid());
-	    
+
 	    if (pwd == NULL) {
 		fprintf(stderr, "%s: cannot get username\n", progname);
 		exit(1);
@@ -309,11 +270,12 @@ main(argc, argv)
 	    username = pwd->pw_name;
 	}
 
-	default_file = (char*)malloc(strlen(MAILPATH) + strlen(username));
+	default_file = (char*)malloc(strlen(XLBIFF_MAILPATH)
+				     + strlen(username));
 	if (default_file == NULL)
 	  ErrExit(True,"default_file malloc()");
 
-	sprintf(default_file,MAILPATH,username);
+	sprintf(default_file,XLBIFF_MAILPATH,username);
 	lbiff_data.file = default_file;
     }
     DP(("file= %s\n",lbiff_data.file));
@@ -403,15 +365,7 @@ NULL};
 |*  Exit  *|  called via callback, exits the program
 \**********/
 void
-#ifdef	FUNCPROTO
 Exit(Widget w, XEvent *event, String *params, Cardinal *num_params)
-#else
-Exit(w, event, params, num_params)
-Widget w;
-XEvent *event;
-String *params;
-Cardinal *num_params;
-#endif
 {
     DP(("++Exit()\n"));
 
@@ -432,41 +386,17 @@ Cardinal *num_params;
 }
 
 
-/************\
-|*  Mailer  *|  called via callback, starts a mailer
-\************/
-void
-#ifdef	FUNCPROTO
-Mailer(Widget w, XEvent *event, String *params, Cardinal *num_params)
-#else
-Mailer(w, event, params, num_params)
-Widget w;
-XEvent *event;
-String *params;
-Cardinal *num_params;
-#endif
-{
-    DP(("++Mailer()\n"));
-
-    if (lbiff_data.mailerCmd != NULL && lbiff_data.mailerCmd[0] != '\0') {
-	Popdown();
-	system(lbiff_data.mailerCmd);
-	Popup();
-	checksize();
-    }
-}
-
-
 /***************\
 |*  checksize  *|  checks mail file to see if new mail is present
 |***************
 |*	This routine stat's the mail spool file and compares its size
-|*	with the previously obtained result.  If the size has become 
+|*	with the previously obtained result.  If the size has become
 |*	zero, it pops down the window.  If nonzero, it calls a routine
 |*	to execute the scanCommand.  If the result of this is non-null,
 |*	it pops up a window showing it (note that users of Berkeley
 |*	mail may have non-empty mail files with all old mail).
 \*/
+void
 checksize()
 {
     static int mailsize = 0;
@@ -505,7 +435,7 @@ checksize()
 
 	if ((p= popen(cmd_buf,"r")) == NULL)
 	  ErrExit(True,"popen(checkCommand)");
-	if (fread(outbuf,1,sizeof outbuf,p) < 0)
+	if (fread(outbuf,1,sizeof outbuf,p) <= 0)
 	  ErrExit(True,"fread(checkCommand)");
 	previous = atol(outbuf);
 	DP(("checkCommand returns %d\n",previous));
@@ -538,7 +468,7 @@ checksize()
 	 **		  _and_ your update interval is too low _and_ you use
 	 **		  a Berkeleyish mailer.  Yuk.
 	 **
-	 ** Doubtless there are errors we should complain about, but this 
+	 ** Doubtless there are errors we should complain about, but this
 	 ** would get too ugly.
 	 */
 	if (stat(lbiff_data.file,&mailstat) != 0) {
@@ -551,7 +481,7 @@ checksize()
     ** If it's changed size, take appropriate action.
     */
     if (mailstat.st_size != mailsize) {
-	DP(("changed size: %d -> %d\n",mailsize,mailstat.st_size));
+        DP(("changed size: %d -> %d\n",mailsize,(int)mailstat.st_size));
 	mailsize = mailstat.st_size;
 	pop_window = True;
     } else if (!visible && lbiff_data.refresh && mailsize != 0) {
@@ -602,18 +532,33 @@ checksize()
 }
 
 
+/************\
+|*  Mailer  *|  called via callback, starts a mailer
+\************/
+void
+Mailer(Widget w, XEvent *event, String *params, Cardinal *num_params)
+{
+    int system_return;
+    DP(("++Mailer()\n"));
+
+    if (lbiff_data.mailerCmd != NULL && lbiff_data.mailerCmd[0] != '\0') {
+	Popdown();
+        system_return = system(lbiff_data.mailerCmd);
+	if (system_return != 0) {
+            fprintf(stderr, "mailer command \"%s\" returned %d (%#x)\n",
+                    lbiff_data.mailerCmd, system_return, system_return);
+        }
+	Popup();
+	checksize();
+    }
+}
+
+
 /*************\
 |*  handler  *|  Checks mail file and reschedules itself to do so again
 \*************/
 void
-#ifdef	FUNCPROTO
 handler( XtPointer closure, XtIntervalId *id )
-#else
-/* ARGSUSED */
-handler( closure, id )
-     XtPointer 	   closure;
-     XtIntervalId  *id;
-#endif
 {
     checksize();
     XtAppAddTimeOut(app_context,lbiff_data.update * 1000, handler, NULL);
@@ -695,17 +640,10 @@ doScan()
 
 
 /****************\
-|*  CheckEvent  *|  
+|*  CheckEvent  *|
 \****************/
 Bool
-#ifdef	FUNCPROTO
 CheckEvent( Display *d, XEvent *e, caddr_t arg )
-#else	/* ~FUNCPROTO */
-CheckEvent( d, e, arg )
-     Display *d;
-     XEvent  *e;
-     caddr_t arg;
-#endif	/* FUNCPROTO */
 {
     if (e->type == MapNotify || e->type == UnmapNotify)
       if (e->xmap.window == (Window)arg)
@@ -726,26 +664,14 @@ static XEvent lastEvent;
 |*  Shrink  *|  get StructureNotify events, popdown if iconified
 \************/
 void
-#ifdef	FUNCPROTO
 Shrink( Widget w, caddr_t data, XEvent *e, Boolean *b )
-#else
-Shrink(w, data, e, b)
-    Widget w;
-    caddr_t data;
-    XEvent *e;
-    Boolean *b;
-#endif
 {
     DP(("++Shrink()\n"));
     if (e->type == MapNotify || e->type == UnmapNotify) {
 	int    event_seen = 0;
 	Window win = e->xmap.window;
 
-#ifdef	USE_BCOPY
-	bcopy((char*)e,(char*)&lastEvent,sizeof(XEvent));
-#else
 	memcpy((char*)&lastEvent,(char*)e,sizeof(XEvent));
-#endif
 
 	XSync(XtDisplay(w),False);
 
@@ -846,12 +772,7 @@ lbiffUnrealize()
 |*  lbiffRealize  *|  reformat window, set the text and bring window up
 \******************/
 void
-#ifdef	FUNCPROTO
 lbiffRealize( char *s )
-#else
-lbiffRealize( s )
-     char *s;
-#endif
 {
     Arg		args[4];
     int		n;
@@ -907,6 +828,7 @@ lbiffRealize( s )
     }
     else {
 	static char	*sound_buf;
+        int		system_return;
 
 	/*
 	** Initialise sound string
@@ -919,7 +841,11 @@ lbiffRealize( s )
 	    sprintf(sound_buf,lbiff_data.sound, lbiff_data.volume);
 	    DP(("---sound= %s\n",sound_buf));
 	}
-	system(sound_buf);
+	system_return = system(sound_buf);
+	if (system_return != 0) {
+            fprintf(stderr, "sound command \"%s\" returned %d (%#x)\n",
+                    sound_buf, system_return, system_return);
+        }
     }
 
     if (lbiff_data.resetSaver)
@@ -933,13 +859,7 @@ lbiffRealize( s )
 |*  getDimensions  *|  get width x height of text string
 \*******************/
 void
-#ifdef	FUNCPROTO
 getDimensions( char *s, Dimension *width, Dimension *height )
-#else
-getDimensions(s,width,height)
-     char *s;
-     Dimension *width, *height;
-#endif
 {
     Dimension	tmp_width;
     int		i,
@@ -984,12 +904,7 @@ getDimensions(s,width,height)
 |*  initStaticData  *|  initializes font size & borderWidth
 \********************/
 void
-#ifdef	FUNCPROTO
 initStaticData( int *bw, int *fontH, int *fontW )
-#else
-initStaticData(bw, fontH, fontW)
-    int *bw, *fontH, *fontW;
-#endif
 {
     Arg		args[2];
     XFontStruct *fs;
@@ -1014,12 +929,7 @@ initStaticData(bw, fontH, fontW)
 |*  toggle_key_led  *|  toggle a keyboard LED on and off
 \********************/
 void
-#ifdef	FUNCPROTO
 toggle_key_led(int flag)
-#else
-toggle_key_led(flag)
-int	flag;
-#endif
 {
     XKeyboardControl	keyboard;
 
@@ -1049,13 +959,7 @@ int	flag;
 |* It is the intention that someday this will bring up a popup window.
 \*/
 void
-#ifdef	FUNCPROTO
 ErrExit(Boolean errno_valid, char *s)
-#else
-ErrExit(errno_valid, s)
-     Boolean errno_valid;
-     char    *s;
-#endif
 {
     if (errno_valid)
       fprintf(stderr,"%s: %s: %s\n", progname, s, strerror(errno));
@@ -1068,28 +972,3 @@ ErrExit(errno_valid, s)
 
     exit(1);
 }
-
-
-#ifdef	NEED_STRERROR
-/**************\
-|*  strerror  *|  return descriptive message text for given errno
-\**************/
-char *
-#ifdef	FUNCPROTO
-strerror(int err)
-#else	/* ~FUNCPROTO */
-strerror(err)
-     int err;
-#endif	/* FUNCPROTO */
-{
-    static char unknown[30];
-    extern int	sys_nerr;
-    extern char *sys_errlist[];
-
-    if (err >= 0 && err < sys_nerr)
-      return sys_errlist[err];
-
-    sprintf(unknown,"Unknown error %d", err);
-    return unknown;
-}
-#endif	/* NEED_STRERROR */
